/* BdfServer_API - Copyright (c) 2012-2024 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.api.instruction;

import fr.exemole.bdfserver.api.configuration.PathConfiguration;
import fr.exemole.bdfserver.api.interaction.Domain;
import fr.exemole.bdfserver.api.roles.Role;
import fr.exemole.bdfserver.tools.configuration.PathConfigurationBuilder;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import net.fichotheque.addenda.Addenda;
import net.fichotheque.addenda.Document;
import net.fichotheque.album.Album;
import net.fichotheque.album.Illustration;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.exportation.access.AccessDef;
import net.fichotheque.exportation.balayage.BalayageDescription;
import net.fichotheque.exportation.scrutari.ScrutariExportDef;
import net.fichotheque.exportation.sql.SqlExportDef;
import net.fichotheque.exportation.table.TableExportDescription;
import net.fichotheque.exportation.transformation.TemplateContentDescription;
import net.fichotheque.exportation.transformation.TemplateDescription;
import net.fichotheque.exportation.transformation.TransformationDescription;
import net.fichotheque.selection.SelectionDef;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.sphere.Sphere;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.RelativePath;


/**
 *
 * @author Vincent Calame
 */
public interface OutputParameters extends BdfParameters {

    public Domain getDomain();

    public String getOutput();

    public RequestMap getRequestMap();

    public BdfCommandResult getBdfCommandResult();

    public default Object getResultObject(String name) {
        BdfCommandResult commandResult = getBdfCommandResult();
        if (commandResult == null) {
            return null;
        }
        return commandResult.getResultObject(name);
    }

    public default Corpus getCorpus() throws ErrorMessageException {
        Corpus corpus = (Corpus) getResultObject(BdfInstructionConstants.CORPUS_OBJ);
        if (corpus == null) {
            FicheMeta ficheMeta = (FicheMeta) getResultObject(BdfInstructionConstants.FICHEMETA_OBJ);
            if (ficheMeta != null) {
                corpus = ficheMeta.getCorpus();
            }
        }
        if (corpus == null) {
            corpus = BdfInstructionUtils.getMandatoryCorpus(getFichotheque(), getRequestMap());
        }
        checkSubsetAccess(corpus);
        return corpus;
    }

    public default Thesaurus getThesaurus() throws ErrorMessageException {
        Thesaurus thesaurus = (Thesaurus) getResultObject(BdfInstructionConstants.THESAURUS_OBJ);
        if (thesaurus == null) {
            Motcle motcle = (Motcle) getResultObject(BdfInstructionConstants.MOTCLE_OBJ);
            if (motcle != null) {
                thesaurus = motcle.getThesaurus();
            }
        }
        if (thesaurus == null) {
            thesaurus = BdfInstructionUtils.getMandatoryThesaurus(getFichotheque(), getRequestMap());
        }
        checkSubsetAccess(thesaurus);
        return thesaurus;
    }

    public default Addenda getAddenda() throws ErrorMessageException {
        Addenda addenda = (Addenda) getResultObject(BdfInstructionConstants.ADDENDA_OBJ);
        if (addenda == null) {
            Document document = (Document) getResultObject(BdfInstructionConstants.DOCUMENT_OBJ);
            if (document != null) {
                addenda = document.getAddenda();
            }
        }
        if (addenda == null) {
            addenda = BdfInstructionUtils.getMandatoryAddenda(getFichotheque(), getRequestMap());
        }
        checkSubsetAccess(addenda);
        return addenda;
    }

    public default Album getAlbum() throws ErrorMessageException {
        Album album = (Album) getResultObject(BdfInstructionConstants.ALBUM_OBJ);
        if (album == null) {
            Illustration illustration = (Illustration) getResultObject(BdfInstructionConstants.ILLUSTRATION_OBJ);
            if (illustration != null) {
                album = illustration.getAlbum();
            }
        }
        if (album == null) {
            album = BdfInstructionUtils.getMandatoryAlbum(getFichotheque(), getRequestMap());
        }
        checkSubsetAccess(album);
        return album;
    }

    public default Sphere getSphere() throws ErrorMessageException {
        Sphere sphere = (Sphere) getResultObject(BdfInstructionConstants.SPHERE_OBJ);
        if (sphere == null) {
            Redacteur redacteur = (Redacteur) getResultObject(BdfInstructionConstants.REDACTEUR_OBJ);
            if (redacteur != null) {
                sphere = redacteur.getSphere();
            }
        }
        if (sphere == null) {
            sphere = BdfInstructionUtils.getMandatorySphere(getFichotheque(), getRequestMap());
        }
        checkSubsetAccess(sphere);
        return sphere;
    }

    public default FicheMeta getFicheMeta() throws ErrorMessageException {
        FicheMeta ficheMeta = (FicheMeta) getResultObject(BdfInstructionConstants.FICHEMETA_OBJ);
        if (ficheMeta == null) {
            ficheMeta = BdfInstructionUtils.getMandatoryFicheMeta(getFichotheque(), getRequestMap(), getBdfUser());
        }
        return ficheMeta;
    }

    public default Illustration getIllustration() throws ErrorMessageException {
        Illustration illustration = (Illustration) getResultObject(BdfInstructionConstants.ILLUSTRATION_OBJ);
        if (illustration == null) {
            illustration = BdfInstructionUtils.getMandatoryIllustration(getFichotheque(), getRequestMap());
        }
        return illustration;
    }

    public default Document getDocument() throws ErrorMessageException {
        Document document = (Document) getResultObject(BdfInstructionConstants.DOCUMENT_OBJ);
        if (document == null) {
            document = BdfInstructionUtils.getMandatoryDocument(getFichotheque(), getRequestMap());
        }
        return document;
    }

    public default Redacteur getRedacteur() throws ErrorMessageException {
        Redacteur redacteur = (Redacteur) getResultObject(BdfInstructionConstants.REDACTEUR_OBJ);
        if (redacteur == null) {
            redacteur = BdfInstructionUtils.getMandatoryRedacteur(getFichotheque(), getRequestMap());
        }
        return redacteur;
    }

    public default Motcle getMotcle() throws ErrorMessageException {
        Motcle motcle = (Motcle) getResultObject(BdfInstructionConstants.MOTCLE_OBJ);
        if (motcle == null) {
            motcle = BdfInstructionUtils.getMandatoryMotcle(getFichotheque(), getRequestMap());
        }
        return motcle;
    }

    public default Role getRole() throws ErrorMessageException {
        Role role = (Role) getResultObject(BdfInstructionConstants.ROLE_OBJ);
        if (role == null) {
            role = BdfInstructionUtils.getMandatoryRole(getBdfServer().getPermissionManager(), getRequestMap());
        }
        return role;
    }

    public default SelectionDef getSelectionDef() throws ErrorMessageException {
        SelectionDef selectionDefDef = (SelectionDef) getResultObject(BdfInstructionConstants.SELECTIONDEF_OBJ);
        if (selectionDefDef == null) {
            selectionDefDef = BdfInstructionUtils.getMandatorySelectionDef(getBdfServer().getSelectionManager(), getRequestMap());
        }
        return selectionDefDef;
    }

    public default ScrutariExportDef getScrutariExportDef() throws ErrorMessageException {
        ScrutariExportDef scrutariExportDef = (ScrutariExportDef) getResultObject(BdfInstructionConstants.SCRUTARIEXPORTDEF_OBJ);
        if (scrutariExportDef == null) {
            scrutariExportDef = BdfInstructionUtils.getMandatoryScrutariExportDef(getBdfServer().getScrutariExportManager(), getRequestMap());
        }
        return scrutariExportDef;
    }

    public default SqlExportDef getSqlExportDef() throws ErrorMessageException {
        SqlExportDef sqlExportDef = (SqlExportDef) getResultObject(BdfInstructionConstants.SQLEXPORTDEF_OBJ);
        if (sqlExportDef == null) {
            sqlExportDef = BdfInstructionUtils.getMandatorySqlExportDef(getBdfServer().getSqlExportManager(), getRequestMap());
        }
        return sqlExportDef;
    }

    public default AccessDef getAccessDef() throws ErrorMessageException {
        AccessDef accessDef = (AccessDef) getResultObject(BdfInstructionConstants.ACCESSDEF_OBJ);;
        if (accessDef == null) {
            accessDef = BdfInstructionUtils.getMandatoryAccessDef(getBdfServer().getAccessManager(), getRequestMap());
        }
        return accessDef;
    }

    public default TableExportDescription getTableExportDescription() throws ErrorMessageException {
        TableExportDescription tableExportDescription = (TableExportDescription) getResultObject(BdfInstructionConstants.TABLEEXPORTDESCRIPTION_OBJ);
        if (tableExportDescription == null) {
            tableExportDescription = BdfInstructionUtils.getMandatoryTableExportDescription(getBdfServer(), getRequestMap());
        }
        return tableExportDescription;
    }

    public default TransformationDescription getTransformationDescription() throws ErrorMessageException {
        TransformationDescription transformationDescription = (TransformationDescription) getResultObject(BdfInstructionConstants.TRANSFORMATIONDESCRIPTION_OBJ);
        if (transformationDescription == null) {
            transformationDescription = BdfInstructionUtils.getMandatoryTransformationDescription(getBdfServer(), getRequestMap());
        }
        return transformationDescription;
    }

    public default TemplateDescription getTemplateDescription() throws ErrorMessageException {
        TemplateDescription templateDescription = (TemplateDescription) getResultObject(BdfInstructionConstants.TEMPLATEDESCRIPTION_OBJ);
        if (templateDescription == null) {
            templateDescription = BdfInstructionUtils.getMandatoryTemplateDescription(getBdfServer(), getRequestMap());
        }
        return templateDescription;
    }

    public default PathConfiguration getPathConfiguration() {
        PathConfiguration pathConfiguration = (PathConfiguration) getResultObject(BdfInstructionConstants.PATHCONFIGURATION_OBJ);
        if (pathConfiguration == null) {
            pathConfiguration = PathConfigurationBuilder.build(getBdfServer());
        }
        return pathConfiguration;
    }

    public default BalayageDescription getBalayageDescription() throws ErrorMessageException {
        BalayageDescription balayageDescription = (BalayageDescription) getResultObject(BdfInstructionConstants.BALAYAGEDESCRIPTION_OBJ);
        if (balayageDescription == null) {
            balayageDescription = BdfInstructionUtils.getMandatoryBalayageDescription(getBdfServer(), getRequestMap());
        }
        return balayageDescription;
    }

    public default TemplateContentDescription getTemplateContentDescription() throws ErrorMessageException {
        TemplateContentDescription templateContentDescription = (TemplateContentDescription) getResultObject(BdfInstructionConstants.TEMPLATECONTENTDESCRIPTION_OBJ);
        if (templateContentDescription == null) {
            TemplateDescription templateDescription = (TemplateDescription) getResultObject(BdfInstructionConstants.TEMPLATEDESCRIPTION_OBJ);
            if (templateDescription == null) {
                templateDescription = BdfInstructionUtils.getMandatoryTemplateDescription(getBdfServer(), getRequestMap());
            }
            templateContentDescription = BdfInstructionUtils.getMandatoryTemplateContentDescription(templateDescription, getRequestMap());
        }
        return templateContentDescription;
    }

    public default BalayageDescription getBalayage() throws ErrorMessageException {
        BalayageDescription balayageDescription = (BalayageDescription) getResultObject(BdfInstructionConstants.BALAYAGEDESCRIPTION_OBJ);
        if (balayageDescription == null) {
            balayageDescription = BdfInstructionUtils.getMandatoryBalayageDescription(getBdfServer(), getRequestMap());
        }
        return balayageDescription;
    }

    public default RelativePath getRelativePath() throws ErrorMessageException {
        RelativePath path = (RelativePath) getResultObject(BdfInstructionConstants.RELATIVEPATH_OBJ);
        if (path == null) {
            path = BdfInstructionUtils.getMandatoryResourcePath(getRequestMap());
        }
        return path;
    }

}
